package com.isu.gaswellwatch.modbus.impl;

import cn.hutool.core.map.MapUtil;
import com.isu.gaswellwatch.dto.modbus.ModbusCommandDto;
import com.isu.gaswellwatch.entity.Response;
import com.isu.gaswellwatch.modbus.CommandService;
import com.isu.gaswellwatch.modbus.data.PersistenceHandler;
import com.isu.gaswellwatch.modbus.data.Redis2DBPersistenceService;
import com.isu.gaswellwatch.service.DeviceOptLogService;
import com.isu.gaswellwatch.utils.HexUtil;
import com.isu.gaswellwatch.vo.command.Command;
import jakarta.annotation.Resource;
import lombok.RequiredArgsConstructor;
import lombok.extern.slf4j.Slf4j;
import org.apache.commons.lang3.ObjectUtils;
import org.apache.commons.lang3.StringUtils;
import org.springframework.core.ParameterizedTypeReference;
import org.springframework.data.redis.core.HashOperations;
import org.springframework.data.redis.core.RedisTemplate;
import org.springframework.http.RequestEntity;
import org.springframework.http.ResponseEntity;
import org.springframework.jdbc.core.JdbcTemplate;
import org.springframework.stereotype.Service;
import org.springframework.web.client.RestTemplate;

import java.util.Collection;
import java.util.HashMap;
import java.util.Map;
import java.util.Objects;

/**
 * @author <a href="mailto:scwsl@foxmail.com">王仕龙</a>
 * 2024/11/27 0:25
 */
@Slf4j
@Service
@RequiredArgsConstructor
public class CommandServiceImpl implements CommandService {

    private final JdbcTemplate jdbcTemplate;
    @Resource(name = "stringRedisTemplate")
    private RedisTemplate<String, String> redisTemplate;
    private final RestTemplate restTemplate = new RestTemplate();
    @Resource
    private DeviceOptLogService deviceOptLogService;

    @Override
    public Response<String> control(Command command) {
        Response<String> result = this.sendCommand("control", command);

        //记录用户保存控制指令日志
        Integer flag = null;
        if (Command.KNPCV1_TURN_ON_THE_WELL.equals(command.getCode())||Command.ETC_TURN_ON_THE_WELL.equals(command.getCode())||
            Command.SCSS_TURN_ON_THE_WELL.equals(command.getCode())) {
            flag = 1;
        } else if (Command.KNPCV1_TURN_OFF_THE_WELL.equals(command.getCode())||
            Command.ETC_TURN_OFF_THE_WELL.equals(command.getCode())||
            Command.SCSS_TURN_OFF_THE_WELL.equals(command.getCode())) {
            flag = 0;
        }
        this.deviceOptLogService.saveGasWellOptLog(flag, command.getDeviceId(),"");
        return result;
    }

    @Override
    public Response<String> collect(Command command) {
        return this.sendCommand("collect", command);
    }

    private Response<String> sendCommand(String type, Command command) {
        try {

            Collection<ModbusCommandDto> modbusCommands = Objects.requireNonNull(command).buildModbusCommand();
            if (ObjectUtils.isEmpty(modbusCommands)) {
                return Response.failed("Modbus command is empty");
            }
            HashOperations<String, String, String> operations = this.redisTemplate.opsForHash();
            Map<String, String> deviceInfoMap = operations.entries(
                    PersistenceHandler.DEVICE_INFO_CACHE + command.getDeviceId());
            if (ObjectUtils.isEmpty(deviceInfoMap)) {
                Map<String, Object> deviceInfo = this.jdbcTemplate.queryForMap(
                        Redis2DBPersistenceService.DEVICE_INFO_SQL + command.getDeviceId());
                if (ObjectUtils.isNotEmpty(deviceInfo)) {
                    deviceInfoMap = new HashMap<>(deviceInfo.size());
                    for (Map.Entry<String, Object> entry : deviceInfo.entrySet()) {
                        deviceInfoMap.put(entry.getKey(),
                                Objects.isNull(entry.getValue()) ? null : String.valueOf(entry.getValue()));
                    }
                    operations.putAll(PersistenceHandler.DEVICE_INFO_CACHE + command.getDeviceId(), deviceInfoMap);
                }
            }

            String identifier;
            long timestamp = System.currentTimeMillis();
            for (ModbusCommandDto item : modbusCommands) {
                if (StringUtils.isBlank(item.getCommand())) {
                    return Response.failed("Modbus command is empty");
                }
                if (item.getLength() < 10) {
                    return Response.failed("Modbus command return message length failed");
                }
                identifier = MapUtil.getStr(deviceInfoMap, "gateway_sn");
                if (StringUtils.isBlank(identifier)) {
                    return Response.failed("Device gateway SN is empty");
                }
                identifier = HexUtil.bytesToHexString(identifier.getBytes());
                item.setKey(StringUtils.joinWith("/", identifier,
                        MapUtil.getStr(deviceInfoMap, "deviceId"),
                        MapUtil.getStr(deviceInfoMap, "commandId"),
                        timestamp));
                item.setIdentifier(identifier);
            }
            RequestEntity<Collection<ModbusCommandDto>> request = RequestEntity
                    .post("http://localhost:9999/modbus-tcp/" + type)
                    .body(modbusCommands);
            ResponseEntity<Response<String>> response = this.restTemplate.exchange(request,
                    new ParameterizedTypeReference<Response<String>>() {
                    });
            if (Objects.isNull(response.getBody())) {
                return Response.failed("Nothing message");
            }
            return response.getBody();
        } catch (Exception e) {
            log.error("Send command fail", e);
            return Response.failed("Send command fail, error: " + e.getMessage());
        }
    }

}
